'Filename   Taster2_103.bas
'Version    1.03
'Date       2013-10-25
'Purpose    Test of "Tasten entprellen - Bulletproof"
'           http://www.mikrocontroller.net/topic/tasten-entprellen-bulletproof
'           Timer0 interrupt is used
'           Test Key_port at 2 different AVR ports
'           One key (here key0) is checked for pressing short/long
'Author     Georg Latzel, DL6GL
'Copyright  (c) Georg Latzel, free for private use under GNU license
'Hardware   ATmega8
'           Crystal 16 MHz
'           2x16 Text LCD @ PortC & D
'           4 keys @ PortC
'Compiler   BASCOM-AVR 2.0.7.3
'           For readable "camel case" variables mark "Don't change case"
'           in BASCOM Options-Programmer-Environment-Editor
'           Flash used: 1460 bytes (17%)
'Edit history
'V1.02      2013-10-25, first release, upgrade of simple V1.00
'V1.03      2013-11-24, keys at different ports

'==============================================================================

'Fuse bits  can be set with AVR Studio 4.0 & ISP-PROG via USB
'           FUSE-Bits (AVR Studio 4.0-Settings, [ ]=no check, [x]=check)
'           In ATMEGA-Data sheets:  Bit = 1 -> not set
'                                   Bit = 0 -> set
'           RSTDISBL   [ ]
'           WTDON      [ ]
'           SPIEN      [x]
'           EESAVE     [ ]
'           BOOTSZ     Boot Flash size=128 words (minimal boot size)
'           BOOTRST    [ ]
'           CKOPT      [ ]
'           BODLEVEL   Brown-out detection at VCC=2.7V
'           BODDEN     [ ]
'           SUT_CKSEL  Ext. Crystal Osc. High Freq., Start-up time 16k + 64ms

$regfile = "m8def.dat"                            'ATmega8
$crystal = 16000000                               '16 MHz xtal
$hwstack = 40
$swstack = 40
$framesize = 40

'Configure LCD,  Ports C & D --------------------------------------------------
'PC.4     LCD-RS
'PC.5     LCD-E
'PD.0     LCD-DB4
'PD.1     LCD-DB5
'PD.3     LCD-DB6
'PD.4     LCD-DB7
Config Lcd = 16 * 2
Config Lcdpin = Pin , Db7 = PortD.4 , Db6 = PortD.3 , Db5 = PortD.1
Config Lcdpin = Pin , Db4 = PortD.0 , E = PortC.5 , Rs = PortC.4
Config Lcdbus = 4

'Configure Key port,  Ports C & D ---------------------------------------------
DDRC.0 = 0                                        'PC.0 = input
DDRC.1 = 0                                        'PC.1 = input
DDRD.6 = 0                                        'PD.6 = input
DDRD.7 = 0                                        'PD.7 = input
PortC.0 = 1                                       'Pullup PC.0
PortC.1 = 1                                       'Pullup PC.1
PortD.6 = 1                                       'Pullup PD.6
PortD.7 = 1                                       'Pullup PD.7

Dim Key_port As Byte
Const Key_sto = 0                                 'Key store @ PINC.0
Const Key_up = 1                                  'Key up    @ PIND.7
Const Key_dwn = 2                                 'Key dwn   @ PIND.6
Const Key_enc = 3                                 'Enc Key   @ PINC.1

'Configure Timer0-Interrupt for Keys (Timer0 = 8bit, 2^8=256 counts) ----------
'Overflow time: Overflow-Counts * Prescale / crystal frequency
'= 255 * 1024 / 16.000.000 = 16,3 msec = ~ 61 Hz (crystal 16 MHz)
'Possible Prescales: 8, 64, 256, 1024, here 1024
Config Timer0 = Timer , Prescale = 1024
Const Presettimer0 = 56                           'Initialize Timer0 (shorten time, >0)
On Timer0 Timer0_isr                              'On Overflow go to Timer0_isr

'Variables Keys ---------------------------------------------------------------
Dim bytIwr0 As Byte
Dim bytKey_ct0 As Byte
Dim bytKey_ct1 As Byte
Dim bytKey_state As Byte
Dim bytKey_press As Byte
Dim bytKey_save As Byte
Dim bytKey_Long As Byte
Dim bitPressedLong As Bit
Dim bytTimer0_runs As Byte

Dim bitKey_up As Bit                              '=1 if key up is pressed
Dim bitKey_dwn As Bit                             '=1 if key down is pressed
Dim bitKey_enc As Bit                             '=1 if key encoder is pressed
Dim bytKey_sto As Byte                            'Sto key pressed: 1(short), 2(long)
Const KeySto_Short = 1                            'Sto key pressed short
Const KeySto_Long = 2                             'Sto key pressed long
Const KeyCounter = 60                             '~ 1 sec, pressed long

Declare Sub ClearLCDLine(byVal bytLine As Byte)

'Initialize Key Data
bytKey_ct0 = 0
bytKey_ct1 = 0
Decr bytKey_ct0
bytKey_ct1 = bytKey_ct0
bytKey_state = 0
bytKey_press = 0
bytKey_Long = 0                                   'Key0 checked for short/long
bytTimer0_runs = 0

Enable Timer0
Enable Interrupts                                 'enable all Interrupts
Timer0 = Presettimer0                             'initialize Timer0

Cls
Cursor Off

Upperline
LCD "Taster2_103"
Lowerline
Lcd "Hello from DL6GL"
Wait 1
Cls

Do                                                'Start of main program ******

   If bytKey_state <> bytKey_save Then            'Any key was pressed
      Disable timer0                              'Stop Timer0 for display
      'Check Key Key_sto --------------
      If bytKey_state.Key_sto <> bytKey_save.Key_sto Then       'Store Key changed
         If bytKey_state.Key_sto = 0 Then         'Key_sto is released
            Call ClearLCDLine(1)
            If bitPressedLong = 0 Then            'Key_sto pressed short
               LCD "Key_sto short"
            Else                                  'Key_sto pressd long
               LCD "Key_sto long"
            End If
         End If
      End If

      'Check Key Key_up -------------
      If bytKey_state.Key_up = 1 Then             'Key_up is pressed
         Call ClearLCDLine(1)
         LCD "Key_up"
      End If

      'Check Key Key_dwn -------------
      If bytKey_state.Key_dwn = 1 Then            'Key_dwn is pressed
         Call ClearLCDLine(1)
         LCD "Key_dwn"
      End If

      'Check Key Key_enc -------------
      If bytKey_state.Key_enc = 1 Then            'Key_enc is pressed
         Call ClearLCDLine(1)
         LCD "Key_enc"
      End If
      Enable Timer0                               'Restart Timer0
   End If

Loop                                              'End of main program *******

End

'==============================================================================
Timer0_isr:
'Interrupt Service Routine for Timer0 (Keys)
'got from Thomas Baer in microcontroller.net
'Original (ASM and C) by Peter Dannegger
'http://www.mikrocontroller.net/topic/tasten-entprellen-bulletproof
'Input:  Key_port       port connected to the keys
'        bytKey_Long    Key number (0...8) to be checked for long/short
'Output: bytKey_press   bit pattern, key which changed from open to pressed
'        bytKey_state   bit pattern, Key state, 1:pressed or 0:not pressed
'                       bits 0...8: keys 0...8 @ Key_port

   Timer0 = Presettimer0
   'Keys ----------------------------------------------------------------------
   'got from Thomas Baer in Microcontroller.net
   bytKey_save = bytKey_state                     'save old status before keys were pressed

   'Keys at different ports
   Key_port.0 = PINC.0                            'Key store   @ PINC.0
   Key_port.1 = PIND.7                            'Key up      @ PIND.7
   Key_port.2 = PIND.6                            'Key down    @ PIND.6
   Key_port.3 = PINC.1                            'Key encoder @ PINC.1

   bytIwr0 = Key_port                             'get Bits of the KeyPort
   bytIwr0 = Not bytIwr0
   bytIwr0 = bytIwr0 Xor bytKey_state
   bytKey_ct0 = bytKey_ct0 And bytIwr0
   bytKey_ct1 = bytKey_ct1 And bytIwr0
   bytKey_ct0 = Not bytKey_ct0
   bytKey_ct1 = bytKey_ct1 Xor bytKey_ct0
   bytIwr0 = bytIwr0 And bytKey_ct0
   bytIwr0 = bytIwr0 And bytKey_ct1
   bytKey_state = bytKey_state Xor bytIwr0
   bytIwr0 = bytIwr0 And bytKey_state
   bytKey_press = bytKey_press Or bytIwr0

   'Check key (bit bytKey_Long in bytKey_state pressed short or long
   If bytKey_state.bytKey_Long = 1 Then           'Key still pressed
      Incr bytTimer0_runs
   Else                                           'Key released
      If bytTimer0_runs >= KeyCounter Then        'Pressed long
         Set bitPressedLong
      Else                                        'Pressed short
         Reset bitPressedLong
      End If
      bytTimer0_runs = 0
   End If

Return

'==============================================================================
Sub ClearLCDLine(byVal bytLine As Byte)
'Clear LCD line bytLine
   Locate bytline , 1
   LCD SPC(16)
   Locate bytline , 1
End Sub